## PyFunge version 0.2 -- http://pyfunge.kldp.net/
## Copyright (c) 2004, Kang Seonghoon (Tokigun).
## This program is distributable under the MIT license.
##
## $Id: execute.py,v 1.11 2004/12/13 05:08:28 tokigun Exp $

"""execute module in PyFunge 0.2 -- http://pyfunge.kldp.net/
This program is distributable under the MIT license.
$Id: execute.py,v 1.11 2004/12/13 05:08:28 tokigun Exp $
"""

from itertools import izip, count
import random, os, sys, time

# PyFunge internal module
from exception import *
import ip, space, fingerprint

class FungeExec:
    is_concurrent = False
    is_filesystem = False

    def __init__(self, platform, argv=None, env=None):
        self.platform = platform
        self.space = self.initspace()
        self.dimension = self.space.dimension
        self.fpregistry = fingerprint.FungeFingerprintRegistry(self)
        self.fpregistry.init()
        self.iplist = {1: ip.FungeIP(self.space, self.fpregistry, 1)}
        self.ipnext = 2
        if argv is None:
            self.argv = sys.argv
        else:
            self.argv = argv
        if env is None:
            self.env = os.environ
        else:
            self.env = env
    
    def initspace(self):
        return None
   
    def command(self, ip):
        if ip.flag & 0x01:
            self.stringmode(ip)
        else:
            instr = ip.getcell()
            try:
                method = ip.commands[instr]
            except KeyError:
                try:
                    method = getattr(self, 'command%d' % instr)
                except AttributeError:
                    self.undefined(ip)
                    return
            method(ip)

    def execute(self, ip):
        self.command(ip)
        ip.pwalk()

    def executeall(self):
        for id in self.iplist.keys():
            try:
                self.execute(self.iplist[id])
            except ExecStopError:
                self.iplist[id].final()
                del self.iplist[id]
            except ExecQuitError:
                self.iplist = {}
                break

    def start(self):
        while self.iplist:
            self.executeall()

class FungeExec_Unefunge93(FungeExec):
    def __init__(self, platform, argv=None, env=None):
        FungeExec.__init__(self, platform, argv, env)
        for id, ip in self.iplist.iteritems():
            ip.delta = [1] + [0] * (ip.dimension - 1)

        # function array for command63()
        self._command63 = [self.command60, self.command62]
        if self.dimension > 1:
            self._command63.extend([self.command94, self.command118])
        if self.dimension > 2:
            self._command63.extend([self.command104, self.command108])
            
        # method wrapping
        for x in xrange(48, 58):
            setattr(self, "command%d" % (x,), self.command48to57)

    def stringmode(self, ip):
        if ip.getcell() == 34:
            self.command34(ip)
        else:
            ip.stack.push(ip.getcell())

    def undefined(self, ip):
        pass
    
    def command32(self, ip):
        pass

    def command33(self, ip):
        if ip.getargs(1)[0]:
            ip.stack.push(0)
        else:
            ip.stack.push(1)

    def command34(self, ip):
        ip.flag ^= 0x01

    def command35(self, ip):
        ip.pwalk()

    def command36(self, ip):
        ip.getargs(1)

    def command37(self, ip):
        b, a = ip.getargs(2)
        if b == 0:
            raise DivisionByZeroError, "Can't divide by zero!"
        else:
            ip.stack.push(a % b)

    def command38(self, ip):
        ip.stack.push(self.platform.getint())

    def command42(self, ip):
        b, a = ip.getargs(2)
        ip.stack.push(a * b)

    def command43(self, ip):
        b, a = ip.getargs(2)
        ip.stack.push(a + b)

    def command44(self, ip):
        self.platform.putchar(ip.getargs(1)[0])

    def command45(self, ip):
        b, a = ip.getargs(2)
        ip.stack.push(a - b)

    def command46(self, ip):
        self.platform.putint(ip.getargs(1)[0])

    def command47(self, ip):
        b, a = ip.getargs(2)
        if b == 0:
            raise DivisionByZeroError, "Can't divide by zero!"
        else:
            ip.stack.push(a // b) # integer division

    def command48to57(self, ip):
        ip.stack.push(ip.getcell() - 48)

    def command58(self, ip):
        try:
            ip.stack.push(ip.stack.peek())
        except StackUnderflowError:
            ip.stack.push(0)

    def command60(self, ip):
        ip.delta = [-1] + [0] * (ip.dimension - 1)

    def command62(self, ip):
        ip.delta = [1] + [0] * (ip.dimension - 1)

    def command63(self, ip):
        random.choice(self._command63)(ip)

    def command64(self, ip):
        raise ExecQuitError, "This program is terminated."

    def command92(self, ip):
        b, a = ip.getargs(2)
        ip.stack.push(b)
        ip.stack.push(a)

    def command95(self, ip):
        if ip.getargs(1)[0]:
            self.command60(ip)
        else:
            self.command62(ip)

    def command96(self, ip):
        b, a = ip.getargs(2)
        if a > b:
            ip.stack.push(1)
        else:
            ip.stack.push(0)

    def command103(self, ip):
        v = ip.getvector()
        try:
            ip.stack.push(ip.space.get(v))
        except (NonDefinedCellError, EmptyCellError):
            ip.stack.push(32)

    def command112(self, ip):
        v = ip.getvector()
        try:
            ip.space.put(v, ip.getargs(1)[0])
        except (NonDefinedCellError, EmptyCellError):
            pass

    def command126(self, ip):
        ip.stack.push(self.platform.getchar())

class FungeExec_Befunge93(FungeExec_Unefunge93):
    def initspace(self):
        return space.FungeStaticSpace(2, (0, 0), (79, 24))
    
    def command94(self, ip):
        ip.delta = [0, -1] + [0] * (ip.dimension - 2)

    def command118(self, ip):
        ip.delta = [0, 1] + [0] * (ip.dimension - 2)

    def command124(self, ip):
        if ip.getargs(1)[0]:
            self.command94(ip)
        else:
            self.command118(ip)

class FungeExec_Unefunge98(FungeExec_Unefunge93):
    def __init__(self, platform, argv=None, env=None):
        FungeExec_Unefunge93.__init__(self, platform, argv, env)

        # method wrapping
        for x in xrange(65, 91):
            setattr(self, "command%d" % (x,), self.command65to90)
        for x in xrange(97, 103):
            setattr(self, "command%d" % (x,), self.command97to102)

    def initspace(self):
        return space.FungeSpace(1)

    def undefined(self, ip):
        self.command114(ip)
        
    def command39(self, ip):
        ip.pwalk()
        ip.stack.push(ip.getcell())

    def command40(self, ip):
        count = ip.getargs(1)[0]
        fpid = reduce(lambda x,y:256*x+y, ip.getargs(count), 0)
        if ip.fingerprint.load(fpid):
            ip.stack.push(fpid)
            ip.stack.push(1)
        else:
            self.command114(ip)

    def command41(self, ip):
        count = ip.getargs(1)[0]
        fpid = reduce(lambda x,y:256*x+y, ip.getargs(count), 0)
        ip.fingerprint.unload(fpid) # result is discarded
    
    def command59(self, ip):
        ip.walk()
        while ip.getcell() != 59:
            ip.walk()

    def command65to90(self, ip):
        self.command114(ip)

    def command97to102(self, ip):
        ip.stack.push(ip.getcell() - 97 + 10)
        
    def command106(self, ip):
        ip.pwalk(ip.stack.pop())
        
    def command107(self, ip):
        ip.pwalk()
        instr = ip.getcell()
        try:
            method = ip.commands[instr]
        except KeyError:
            try:
                method = getattr(self, 'command%d' % instr)
            except AttributeError:
                return
        for x in xrange(ip.getargs(1)[0]):
            method(ip)
    
    def command110(self, ip):
        ip.stack.clear()
        
    def command113(self, ip):
        raise ExecQuitError, "This program is terminated."
        
    def command114(self, ip):
        for i in xrange(ip.dimension):
            ip.delta[i] = -ip.delta[i]
        
    def command115(self, ip):
        current = ip.getargs(1)[0]
        ip.pwalk()
        ip.space.put(ip.position, current)
        ip.pwalk(-1)
        
    def command117(self, ip):
        # NOTE: this function doesn't use FungeStack API, but uses its structure
        #       directly. so it should be modified when FungeStack is modified.
        
        if ip.stack.slen() < 2:
            self.command114(ip)
        else:
            n = ip.getargs(1)[0]
            if n > 0:
                if n < ip.stack.stack[-2].len():
                    ip.stack.stack[-1].stack.extend([0] * (n - ip.stack.stack[-2].len()))
                    ip.stack.stack[-2].reverse()
                    ip.stack.stack[-1].stack.extend(ip.stack.stack[-2])
                    del ip.stack.stack[-2].stack[:]
                else:
                    buffer = ip.stack.stack[-2].stack[-n:]
                    buffer.reverse()
                    ip.stack.stack[-1].stack.extend(buffer)
                    del ip.stack.stack[-2].stack[-n:]
            elif n > 0:
                if abs(n) < ip.stack.stack[-1].len():
                    ip.stack.stack[-2].stack.extend([0] * (abs(n) - ip.stack.stack[-1].len()))
                    ip.stack.stack[-1].reverse()
                    ip.stack.stack[-2].stack.extend(ip.stack.stack[-1])
                    del ip.stack.stack[-1].stack[:]
                else:
                    buffer = ip.stack.stack[-1].stack[-abs(n):]
                    buffer.reverse()
                    ip.stack.stack[-2].stack.extend(buffer)
                    del ip.stack.stack[-1].stack[-abs(n):]
        
    def command120(self, ip):
        ip.delta = ip.getvector()
        
    def command121(self, ip):
        mode = ip.stack.pop()
        toss = ip.stack.stack[-1].len()
        tm = time.localtime()
        ip.space.updaterect()

        # 20th item: enviroment variables
        ip.stack.push(0)
        for k, v in self.env.iteritems():
            ip.stack.push_string("%s=%s" % (k,v))
        # 19th item: command line arguments
        ip.stack.push(0)
        ip.stack.push(0)
        for cmd in self.argv[::-1]:
            ip.stack.push_string(cmd)
        # 18th item: size of each stack in stack stack
        for item in ip.stack.stack[:-1]:
            ip.stack.push(item.len())
        ip.stack.push(toss)
        # 17th item: size of stack stack
        ip.stack.push(ip.stack.slen())
        # 16th item: time
        ip.stack.push((tm[3] << 16) + (tm[4] << 8) + tm[5])
        # 15th item: date
        ip.stack.push(((tm[0]-1900) << 16) + (tm[1] << 8) + tm[2])
        # 14th item: greatest point (relative to the least point)
        for x in xrange(ip.dimension):
            ip.stack.push(ip.space.rectmax[x] - ip.space.rectmin[x])
        # 13th item: least point (relative to origin)
        for x in xrange(ip.dimension):
            ip.stack.push(ip.space.rectmin[x])
        # 12th item: storage offset
        ip.stack.push_list(ip.offset)
        # 11th item: delta
        ip.stack.push_list(ip.delta)
        # 10th item: position
        ip.stack.push_list(ip.position)
        # 9th item: team number (concurrent funge)
        ip.stack.push(ip.th_team)
        # 8th item: unique id (concurrent funge)
        ip.stack.push(ip.th_id)
        # 7th item: number of dimensions
        ip.stack.push(self.dimension)
        # 6th item: path seperator
        ip.stack.push(ord(os.sep))
        # 5th item: operating paradigm (#todo; also it can be 2.)
        ip.stack.push(1)
        # 4th item: implementation's version
        ip.stack.push(200)
        # 3rd item: implementation's handprint
        ip.stack.push(0x50794698) # "PyF\x98"
        # 2nd item: number of bytes per cell
        ip.stack.push(4)
        # 1st item: flags (#todo)
        flag = 0x00
        if self.is_concurrent:
            flag += 0x01
        if self.is_filesystem: # it will be support "i", "o", "=".
            flag += 0x0E
        if self.platform.is_unbufferedio:
            flag += 0x10
        ip.stack.push(flag)

        if mode > 0:
            value = ip.stack.peek(mode)
            infolen = ip.stack.len() - toss
            del ip.stack.stack[-1].stack[-infolen:] #todo?
    
    def command122(self, ip):
        pass
    
    def command123(self, ip):
        # NOTE: this function doesn't use FungeStack API, but uses its structure
        #       directly. so it should be modified when FungeStack is modified.
        
        n = ip.getargs(1)[0]
        ip.stack.spush()
        if n >= ip.stack.stack[-2].len():
            ip.stack.stack[-1].stack.extend(ip.stack.stack[-2].stack[-n:])
            del ip.stack.stack[-2].stack[-n:]
        elif n > 0:
            ip.stack.stack[-1].stack.extend([0] * (n - ip.stack.stack[-2].len()))
            ip.stack.stack[-1].stack.extend(ip.stack.stack[-2].stack)
            del ip.stack.stack[-2].stack[:]
        elif n < 0:
            ip.stack.push_list([0] * abs(n))
        for x in ip.offset:
            ip.stack.stack[-2].push(x)
        ip.pwalk()
        ip.offset = ip.position
        ip.pwalk(-1)
    
    def command125(self, ip):
        # NOTE: this function doesn't use FungeStack API, but uses its structure
        #       directly. so it should be modified when FungeStack is modified.
        
        if ip.stack.slen() < 2:
            self.command114(ip)
        else:
            n = ip.getargs(1)[0]
            ip.offset = list(ip.getvector())
            if n >= ip.stack.len():
                ip.stack.stack[-2].stack.extend(ip.stack.stack[-1].stack[-n:])
            elif n > 0:
                ip.stack.stack[-2].stack.extend([0] * (n - ip.stack.stack[-2].len()))
                ip.stack.stack[-2].stack.extend(ip.stack.stack[-1].stack)
            elif n < 0:
                del ip.stack.stack[-2].stack[-abs(n):]
            ip.stack.spop()
    
    def execute(self, ip):
        self.command(ip)
        ip.walk()
    
class FungeExec_Befunge98(FungeExec_Unefunge98, FungeExec_Befunge93):
    def initspace(self):
        return space.FungeSpace(2)

    def command91(self, ip):
        ip.delta[0], ip.delta[1] = -ip.delta[1], ip.delta[0]
        
    def command93(self, ip):
        ip.delta[0], ip.delta[1] = ip.delta[1], -ip.delta[0]
        
    def command119(self, ip):
        b, a = ip.getargs(2)
        if a < b:
            self.command91(ip)
        elif a > b:
            self.command93(ip)
        else:
            self.command122(ip)

class FungeExec_Trefunge98(FungeExec_Befunge98):
    def initspace(self):
        return space.FungeSpace(3)

    def command104(self, ip):
        ip.delta = [0, 0, -1] + [0] * (ip.dimension - 3)
        
    def command108(self, ip):
        ip.delta = [0, 0, 1] + [0] * (ip.dimension - 3)
        
    def command109(self, ip):
        if ip.getargs(1)[0]:
            self.command104(ip)
        else:
            self.command108(ip)

class FungeExec_Filesystem98:
    is_filesystem = True

    def command61(self, ip):
        command = ip.stack.pop_string()
        ip.stack.push(os.system(command))
    
    def command105(self, ip):
        filename = ip.stack.pop_string()
        flag = ip.getargs(1)
        position = ip.getvector()

        try:
            handle = file(filename, "r")
            contents = ''.join(handle.readlines())
            handle.close()

            if flag & 0x01:
                xpos = 0
                for x, char in izip(count(position[0]), contents):
                    ip.space.put((x,) + position[1:], ord(x))
                else:
                    size = (x - position[0] + 1,) + (1,) * (ip.dimension - 1)
            else:
                size = ip.space.putspace(position, contents)
            
            ip.stack.push_list(size)
            ip.stack.push_list(position)
        except IOError:
            self.command114(ip)
    
    def command111(self, ip):
        filename = ip.stack.pop_string()
        flag = ip.getargs(1)
        position = ip.getvector()
        size = ip.getvector()

        if flag & 0x01:
            trimmed = True
        else:
            trimmed = False

        contents = ip.space.getspace(position, size, trimmed)

        try:
            handle = file(filename, "w")
            handle.write(contents)
            handle.close()
        except IOError:
            self.command114(ip)

class FungeExec_Concurrent98:
    is_concurrent = True
    
    def command64(self, ip):
        raise ExecStopError, "This IP has stopped."
    
    def command116(self, ip):
        clone = ip.clone(self.ipnext)
        clone.pwalk()
        self.iplist[self.ipnext] = clone
        self.ipnext += 1

#################################################################################

class FungeExec_FUnefunge98(FungeExec_Filesystem98, FungeExec_Unefunge98): pass
class FungeExec_FBefunge98(FungeExec_Filesystem98, FungeExec_Befunge98): pass
class FungeExec_FTrefunge98(FungeExec_Filesystem98, FungeExec_Trefunge98): pass
class FungeExec_CUnefunge98(FungeExec_Concurrent98, FungeExec_Unefunge98): pass
class FungeExec_CBefunge98(FungeExec_Concurrent98, FungeExec_Befunge98): pass
class FungeExec_CTrefunge98(FungeExec_Concurrent98, FungeExec_Trefunge98): pass
class FungeExec_FCUnefunge98(FungeExec_Filesystem98, FungeExec_Concurrent98, FungeExec_Unefunge98): pass
class FungeExec_FCBefunge98(FungeExec_Filesystem98, FungeExec_Concurrent98, FungeExec_Befunge98): pass
class FungeExec_FCTrefunge98(FungeExec_Filesystem98, FungeExec_Concurrent98, FungeExec_Trefunge98): pass

