## PyFunge version 0.2 -- http://pyfunge.kldp.net/
## Copyright (c) 2004, Kang Seonghoon (Tokigun).
## This program is distributable under the MIT license.
##
## $Id: fingerprint.py,v 1.4 2004/12/13 05:08:28 tokigun Exp $

"""fingerprint module in PyFunge 0.2 -- http://pyfunge.kldp.net/
This program is distributable under the MIT license.
$Id: fingerprint.py,v 1.4 2004/12/13 05:08:28 tokigun Exp $
"""

import os, os.path, imp, sys, glob, copy

# PyFunge internal module
import ip

MODULE_PREFIX = '__fingerprint__'
MODULE_API = 'PyFungeAPIv1'

class FungeFingerprintRegistry:
    def __init__(self, exc):
        self.exc = exc
        self.registry = {}

        sys.modules[MODULE_PREFIX] = imp.new_module(MODULE_PREFIX)
    
    def register(self, path, name):
        try:
            info = imp.find_module(name, [path])
            if name[0].isalpha() or name[0] == '_':
                pname = MODULE_PREFIX + '.' + name
            else:
                pname = MODULE_PREFIX + '._' + name
            sys.path.append(path)
            mod = imp.load_module(pname, *info)
        except ImportError:
            sys.path.remove(path)
            return False
        else:
            sys.path.remove(path)
        
        try:
            if mod.__api__ != MODULE_API:
                del mod
                del sys.modules[pname]
                return False
            self.registry.update(mod.__fingerprints__)
        except: return False

        try: mod.init()
        except: pass
        
        return True

    def init(self):
        libpath = os.path.split(__file__)[0]
        fppath = os.path.join(libpath, "fingerprint")
        sys.path.append(libpath)
        for f in glob.glob(os.path.join(fppath, 'fp*.py')):
            self.register(*os.path.split(f[:-3]))
        sys.path.remove(libpath)

    def final(self):
        for mod in sys.modules.keys():
            if mod.startswith(MODULE_PREFIX):
                try:
                    if sys.modules[mod] is not None:
                        sys.modules[mod].final()
                    del sys.modules[mod]
                except:
                    pass

        self.registry = {}


class FungeFingerprint:
    def __init__(self, registry, ip):
        self.ip = ip
        self.reg = registry
        self.fingerprints = []

    def clone(self):
        clone = FungeFingerprint(self.registry.exc, self.ip)
        for fp in self.fingerprints:
            clone.fingerprints.append(copy.deepcopy(fp))
        return clone

    def final(self):
        while self.fingerprints:
            fp = self.fingerprints.pop()
            fp.final()
            del fp
        
    def load(self, id):
        try:
            fp = self.reg.registry[id]
            fpobj = fp(self.reg.exc, self.ip)
            self.fingerprints.append(fpobj)
            fpobj.init()
            return True
        except:
            return False

    def unload(self, id):
        try:
            fp = self.reg[id]
        except:
            return False
        
        fps = self.fingerprints
        for idx in xrange(len(fps)-1, -1, -1):
            if fps[idx].__class__ == fp:
                fps[idx].final()
                del fps[idx]
                return True
        else:
            return False


class FungeFingerprintBase:

    def __init__(self, exc, ip):
        self.exc = exc
        self.ip = ip

    def __getattr__(self, name):
        return getattr(self.exc, name)

    def init(self):
        self.prevcmd = {}
        for meth in dir(self):
            if meth.startswith('command'):
                instr = int(meth[7:])
                self.prevcmd[instr] = self.ip.commands.get(instr, None)
                self.ip.commands[instr] = getattr(self, meth)
    
    def final(self):
        for instr, func in self.prevcmd.iteritems():
            if func is not None:
                self.ip.commands[instr] = func


'''
# currently not implemented
class FungeDynamicFingerprint(ip.FungeIP, FungeFingerprintBase):
    
    def __init__(self):
        pass

    def bind(self):
        pass

    def unbind(self):
        pass
'''

