## PyFunge version 0.2 -- http://pyfunge.kldp.net/
## Copyright (c) 2004, Kang Seonghoon (Tokigun).
## This program is distributable under the MIT license.
##
## $Id: ip.py,v 1.6 2004/12/13 05:08:28 tokigun Exp $

"""ip module in PyFunge 0.2 -- http://pyfunge.kldp.net/
This program is distributable under the MIT license.
$Id: ip.py,v 1.6 2004/12/13 05:08:28 tokigun Exp $
"""

import copy, os.path

# PyFunge internal module
from exception import *
import space, stack, fingerprint

class FungeIP:
    def __init__(self, fspace, fpreg, id = 0):
        self.dimension = fspace.dimension
        self.position = [0] * fspace.dimension
        self.delta = [0] * fspace.dimension
        self.offset = [0] * fspace.dimension
        self.flag = 0x0
        self.space = fspace
        self.stack = stack.FungeSStack()
        self.commands = {}
        self.fingerprint = fingerprint.FungeFingerprint(fpreg, self)
        self.th_team = 1 # unused
        self.th_id = id
        
        if isinstance(self.space, space.FungeStaticSpace):
            self.walk = self.walk_static
            self.pwalk = self.walk_static
        else:
            self.walk = self.walk_dynamic
            self.pwalk = self.pwalk_dynamic

    def clone(self, id = 0):
        clone = FungeIP(self.space, self.space)
        clone.dimension = self.dimension
        clone.position = self.position[:]
        clone.delta = [-x for x in self.delta]
        clone.offset = self.offset[:]
        clone.flag = self.flag
        clone.stack = copy.deepcopy(self.stack)
        clone.commands = self.commands.copy()
        clone.fingerprint = copy.copy(self.fingerprint)
        clone.th_team = self.th_team
        clone.th_id = id or self.th_id

        return clone

    def final(self):
        self.fingerprint.final()

    def getcell(self):
        try:
            return self.space.get(self.position)
        except (NonDefinedCellError, EmptyCellError):
            return 32

    def getargs(self, number):
        return tuple(
            [not self.stack.is_empty() and self.stack.pop() or 0
             for x in xrange(number)]
        )

    def getvector(self):
        return self.getargs(self.dimension)[::-1]
    
    def walk_dynamic(self, offset = 1):
        for i in xrange(self.dimension):
            self.position[i] += offset * self.delta[i]
            if self.position[i] > self.space.rectmax[i]:
                self.position[i] = self.space.rectmin[i] + \
                    (self.position[i] - self.space.rectmin[i]) % self.delta[i]
            elif self.position[i] < self.space.rectmin[i]:
                self.position[i] = self.space.rectmax[i] - \
                    (self.space.rectmax[i] - self.position[i]) % self.delta[i]

    def pwalk_dynamic(self, offset = 1):
        for i in xrange(self.dimension):
            self.position[i] += offset * self.delta[i]

    def walk_static(self, offset = 1):
        for i in xrange(self.dimension):
            self.position[i] = self.space.rectmin[i] + \
                (self.position[i] + offset * self.delta[i] - self.space.rectmin[i]) % \
                (self.space.rectmax[i] - self.space.rectmin[i] + 1)

