## PyFunge version 0.2 -- http://pyfunge.kldp.net/
## Copyright (c) 2004, Kang Seonghoon (Tokigun).
## This program is distributable under the MIT license.
##
## $Id: platform.py,v 1.5 2004/12/12 06:34:40 tokigun Exp $

"""platform module in PyFunge 0.2 -- http://pyfunge.kldp.net/
This program is distributable under the MIT license.
$Id: platform.py,v 1.5 2004/12/12 06:34:40 tokigun Exp $
"""

class FungePlatform:
    is_unbufferedio = False
    
    def __init__(self):
        self.getbuffer = ''

    # unbuffered input/output (not implemented)
    def getchar(self):
        raise NotImplementedError
    def putchar(self, char):
        raise NotImplementedError

    # buffered input/output
    def _getline(self):
        return raw_input() + '\n'
    def _getchar(self):
        if self.getbuffer == '':
            self.getbuffer = self._getline()
        char, self.getbuffer = self.getbuffer[0], self.getbuffer[1:]
        return ord(char)
    def _putchar(self, char):
        self.putchar(char)

    # buffered input/output with format
    def getint(self):
        str = ''
        flag = 1
        while not str.isdigit():
            if str == '-':
                flag = -flag
            elif str != '+':
                flag = 1
            str = chr(self._getchar())
        while str.isdigit():
            str += chr(self._getchar())
        return flag * int(str[:-1])
    def putint(self, int):
        for x in str(int) + ' ':
            self.putchar(ord(x))

class FungePlatform_posix(FungePlatform):
    is_unbufferedio = True
    
    def __init__(self):
        FungePlatform.__init__(self)
        import curses
        self.screen = curses.initscr()
    def __del__(self):
        import curses
        curses.endwin()
        del curses
    
    # unbuffered input/output
    def getchar(self):
        return self.screen.getch()
    def putchar(self, char):
        self.screen.echochar(char)

class FungePlatform_nt(FungePlatform):
    is_unbufferedio = True
    
    def __init__(self):
        FungePlatform.__init__(self)

    # unbuffered input/output
    def getchar(self):
        import msvcrt
        return ord(msvcrt.getche())
    def putchar(self, char):
        import msvcrt
        msvcrt.putch(chr(char))

#################################################################################
## The following classes are unimplemented... :(

class FungePlatform_mac(FungePlatform):
    def __init__(self): raise NotImplementedError
class FungePlatform_os2(FungePlatform):
    def __init__(self): raise NotImplementedError
class FungePlatform_ce(FungePlatform):
    def __init__(self): raise NotImplementedError
class FungePlatform_java(FungePlatform):
    def __init__(self): raise NotImplementedError
class FungePlatform_riscos(FungePlatform):
    def __init__(self): raise NotImplementedError

