## PyFunge version 0.2 -- http://pyfunge.kldp.net/
## Copyright (c) 2004, Kang Seonghoon (Tokigun).
## This program is distributable under the MIT license.
##
## $Id: space.py,v 1.5 2004/12/12 06:34:40 tokigun Exp $

"""space module in PyFunge 0.2 -- http://pyfunge.kldp.net/
This program is distributable under the MIT license.
$Id: space.py,v 1.5 2004/12/12 06:34:40 tokigun Exp $
"""

from itertools import izip, count

# PyFunge internal module
from exception import *

class FungeSpace:
    def __init__(self, dimension):
        if not isinstance(dimension, int):
            raise NumberOfDimensionError, "Number of dimensions should be integer."
        if dimension < 1:
            raise NumberOfDimensionError, "Lahey-space can't have less than one dimension."
        
        self.dimension = dimension
        self.space = {}
        self.rectmax = []
        self.rectmin = []

    def check_position(self, pos):
        if isinstance(pos, list):
            pos = tuple(pos)
        elif not isinstance(pos, tuple):
            raise NonDefinedCellError, "Unknown position vector."
        if len(pos) != self.dimension:
            raise NonDefinedCellError, "Requested cell is non-defined cell."
        for coord in pos:
            if not isinstance(coord, int):
                raise NonDefinedCellError, "Component of position vector is not integer."

        return pos

    def get(self, pos):
        pos = self.check_position(pos)
        if self.space.has_key(pos):
            return self.space[pos]
        else:
            raise EmptyCellError, "Requested cell is empty."

    def put(self, pos, char):
        pos = self.check_position(pos)
        if char == 32:
            try:
                del self.space[pos]
            except:
                pass
        else:
            self.space[pos] = char

            if len(self.rectmax) == 0:
                self.rectmax = list(pos)
            else:
                for i in xrange(self.dimension):
                    if self.rectmax[i] < pos[i]:
                        self.rectmax[i] = pos[i]

            if len(self.rectmin) == 0:
                self.rectmin = list(pos)
            else:
                for i in xrange(self.dimension):
                    if self.rectmin[i] > pos[i]:
                        self.rectmin[i] = pos[i]

    def getspace(self, pos, size, rtrim = True):
        pos = self.check_position(pos)
        if len(size) > 3:
            raise RuntimeError, "Currently PyFunge doesn't support more than 3D funge-space."
        elif len(size) > 2:
            str = "\f".join([
                self.getspace(pos[:2] + (z,) + pos[3:], size[:2], rtrim)
                for z in xrange(pos[2], pos[2] + size[2])
            ])
            if rtrim:
                str = str.rstrip("\f")
            return str
        elif len(size) > 1:
            str = "\n".join([
                self.getspace(pos[:1] + (y,) + pos[2:], size[:1], rtrim)
                for y in xrange(pos[1], pos[1] + size[1])
            ])
            if rtrim:
                str = str.rstrip("\n")
            return str
        else:
            str = ""
            for x in xrange(pos[0], pos[0] + size[0]):
                try:
                    str += chr(self.get((x,) + pos[1:]))
                except (EmptyCellError, NonDefinedCellError):
                    str += " "
                except ValueError:
                    str += "?"
            if rtrim:
                return str.rstrip(" ")
            else:
                return str

    def putspace(self, pos, str):
        pos = self.check_position(pos)
        if len(pos) < 3:
            pos = pos + (0,) * (3 - len(pos))
        maxrect = list(pos)
        for z, plain in izip(count(pos[2]), str.split("\f")):
            for y, line in izip(count(pos[1]), plain.splitlines()):
                for x, char in izip(count(pos[0]), line):
                    self.put(((x, y, z) + pos[3:])[:self.dimension], ord(char))
                else:
                    maxrect[0] = x + 1
            else:
                maxrect[1] = y + 1
        else:
            maxrect[2] = z + 1

        for x in xrange(self.dimension):
            maxrect[x] -= pos[x]
        return maxrect[:self.dimension]
    
    def updaterect(self):
        dimension = self.dimension
        rectmin = [0] * dimension
        rectmax = [0] * dimension
        for pos in self.space:
            for i in xrange(dimension):
                if rectmin[i] > pos[i]:
                    rectmin[i] = pos[i]
                elif rectmax[i] < pos[i]:
                    rectmax[i] = pos[i]

        self.rectmin = rectmin
        self.rectmax = rectmax
        return (rectmin, rectmax)

class FungeStaticSpace(FungeSpace):
    def __init__(self, dimension, rectmin, rectmax):
        FungeSpace.__init__(self, dimension)

        FungeSpace.check_position(self, rectmin)
        FungeSpace.check_position(self, rectmax)
        self.rectmin = rectmin
        self.rectmax = rectmax

    def check_position(self, pos):
        pos = FungeSpace.check_position(self, pos)
        for x in xrange(0, self.dimension):
            if self.rectmin[x] <= pos[x] <= self.rectmax[x]:
                pass
            else:
                raise NonDefinedCellError, "Requested cell is in outside of space."

        return pos
    
    def updaterect(self):
        pass

class Befunge93Space(FungeStaticSpace):
    def __init__(self):
        FungeStaticSpace.__init__(self, 2, (0, 0), (79, 24))

