## PyFunge version 0.2 -- http://pyfunge.kldp.net/
## Copyright (c) 2004, Kang Seonghoon (Tokigun).
## This program is distributable under the MIT license.
##
## $Id: stack.py,v 1.5 2004/12/12 06:34:40 tokigun Exp $

"""stack module in PyFunge 0.2 -- http://pyfunge.kldp.net/
This program is distributable under the MIT license.
$Id: stack.py,v 1.5 2004/12/12 06:34:40 tokigun Exp $
"""

# PyFunge internal module
from exception import *

class FungeStack:
    def __init__(self):
        self.stack = []
        self.mode_invert = False
        self.mode_queue = False

    def __copy__(self):
        clone = FungeStack()
        clone.stack = self.stack
        clone.mode_invert = self.mode_invert
        clone.mode_queue = self.mode_queue

        return clone
    
    def push(self, item):
        if self.mode_invert:
            self.stack.insert(0, item)
        else:
            self.stack.append(item)
        
    def pop(self):
        if len(self.stack) > 0:
            if self.mode_queue:
                temp = self.stack[0]
                del self.stack[0]
                return temp
            else:
                return self.stack.pop()
        else:
            raise StackUnderflowError, "There is no item to pop."

    def peek(self, offset = 0):
        if len(self.stack) > 0:
            return self.stack[-1-offset]
        else:
            raise StackUnderflowError, "There is no item to peek."

    def clear(self):
        self.stack = []

    def is_empty(self):
        return (len(self.stack) == 0)

    def len(self):
        return len(self.stack)

class FungeSStack:
    def __init__(self):
        self.stack = [FungeStack()]

    def __copy__(self):
        clone = FungeSStack()
        clone.stack = self.stack[:]

        return clone
    
    def spush(self):
        self.stack.append(FungeStack())

    def spop(self):
        if len(self.stack) > 1:
            return self.stack.pop()
        else:
            raise SStackUnderflowError, "Now stack stack has only one stack."

    def speek(self):
        return self.stack[-1]

    def slen(self):
        return len(self.stack)

    def toggle_invert(self):
        # potentional warning -- different status flag among stacks in the stack stack.
        self.stack[-1].mode_invert = not self.stack[-1].mode_invert
        return self.stack[-1].mode_invert
    
    def toggle_queue(self):
        # potentional warning -- different status flag among stacks in the stack stack.
        self.stack[-1].mode_queue = not self.stack[-1].mode_queue
        return self.stack[-1].mode_queue

    def push(self, item):
        self.stack[-1].push(item)

    def pop(self):
        return self.stack[-1].pop()

    def peek(self, offset = 0):
        return self.stack[-1].peek(offset)

    def clear(self):
        self.stack[-1].stack = []
    
    def is_empty(self):
        return (len(self.stack[-1].stack) == 0)

    def len(self):
        return len(self.stack[-1].stack)

    def push_string(self, strn):
        if not isinstance(strn, str):
            raise TypeError, "Can't push non-string variable into the stack!"
        else:
            self.push(0)
            for x in xrange(len(strn)-1, -1, -1):
                self.push(ord(strn[x]))

    def pop_string(self):
        str = ""
        try:
            while self.peek() != 0:
                str += chr(self.pop())
            self.pop()
        except StackUnderflowError:
            pass

        return str

    def push_list(self, lst):
        if isinstance(lst, (list, tuple)):
            for x in lst:
                self.push(x)
        else:
            raise TypeError, "Can't push this list!"

    def pop_list(self, length): #todo?
        pass

