## PyFunge version 0.2 -- http://pyfunge.kldp.net/
## Copyright (c) 2004, Kang Seonghoon (Tokigun).
## This program is distributable under the MIT license.
##
## $Id: fp_standard.py,v 1.1 2004/12/12 06:45:05 tokigun Exp $

"""PyFunge 0.2 fingerprint: Funge-98 Standard Fingerprint -- http://pyfunge.kldp.net/
This program is distributable under the MIT license.
$Id: fp_standard.py,v 1.1 2004/12/12 06:45:05 tokigun Exp $
"""

__api__ = 'PyFungeAPIv1'

from fingerprint import FungeFingerprintBase

class FingerprintNULL(FungeFingerprintBase):
    def init(self):
        self.prevcmd = {}
        for instr, func in self.ip.commands.iteritems():
            if 65 <= instr <= 90:
                self.prevcmd[instr] = func
                del self.ip.commands[instr]

    def final(self):
        self.ip.commands.update(self.prevcmd)

class FingerprintROMA(FungeFingerprintBase):
    def command73(self, ip): ip.stack.push(1)
    def command86(self, ip): ip.stack.push(5)
    def command88(self, ip): ip.stack.push(10)
    def command76(self, ip): ip.stack.push(50)
    def command67(self, ip): ip.stack.push(100)
    def command68(self, ip): ip.stack.push(500)
    def command77(self, ip): ip.stack.push(1000)

class FingerprintORTH(FungeFingerprintBase):
    def command65(self, ip):
        b, a = ip.getargs(2)
        ip.stack.push(a & b)

    def command79(self, ip):
        b, a = ip.getargs(2)
        ip.stack.push(a | b)

    def command69(self, ip):
        b, a = ip.getargs(2)
        ip.stack.push(a ^ b)

    def command88(self, ip):
        ip.position[0] = ip.getargs(1)[0] - ip.delta[0]

    def command89(self, ip):
        if ip.dimension > 1:
            ip.position[1] = ip.getargs(1)[0] - ip.delta[1]
        else:
            self.exc.command114()

    def command86(self, ip):
        ip.delta[0] = ip.getargs(1)[0]

    def command87(self, ip):
        if ip.dimension > 1:
            ip.delta[1] = ip.getargs(1)[0]
        else:
            self.command114(ip)

    def command71(self, ip):
        v = ip.getvector()[::-1]
        try:
            ip.stack.push(ip.space.get(v))
        except (NonDefinedCellError, EmptyCellError):
            ip.stack.push(32)

    def command80(self, ip):
        v = ip.getvector()
        try:
            ip.space.put(v, ip.getargs(1)[0])
        except (NonDefinedCellError, EmptyCellError):
            pass

    def command90(self, ip):
        if ip.getargs(1)[0]:
            self.command122(ip)
        else:
            self.command35(ip)

    def command83(self, ip):
        c = ip.getargs(1)[0]
        while c:
            self.platform.putchar(c)
            c = ip.getargs(1)[0]

__fingerprints__ = {
    0x4e554c4c: FingerprintNULL,
    0x4f525448: FingerprintORTH,
    0x524f4d41: FingerprintROMA,
}

